from time import sleep
from pyee import EventEmitter

from .manager import Manager


class LocoRobo(EventEmitter):

    def __init__(self):
        super(LocoRobo, self).__init__()
        self.manager = Manager()

        self.manager.on('error', lambda error: self.emit('error', error))
        self.manager.on('robots_found', lambda robots_found: self.emit('robots_found', robots_found))
        self.manager.on('connected', lambda robot: self.emit('connected', robot))
        self.manager.on('disconnected', lambda robot: self.emit('disconnected', robot))
        self.manager.on('close', lambda: self.emit('close'))

    def setup(self, address):
        return self.start(address)

    def start(self, address):
        self.manager.start(address)

    def stop(self):
        self.manager.stop()

    def reset_ble(self):
        self.manager.reset_ble()

    def scan(self, timeout):
        self.manager.start_scanning(timeout)

        return self.manager.scanning_robots

    def wait(self, time):
        sleep(time)

    def start_scanning(self, timeout):
        return self.scan(timeout)

    def get_robots(self, return_type=object):
        if return_type is not object:
            robots = []

            for value in self.manager.scanning_robots.itervalues():
                robots.append(value)

        else:
            robots = self.manager.scanning_robots

        return robots
