import time
from pyee import EventEmitter

from .exceptions import APIError
from .control_codes import OutgoingUSBMessageType
from .serial_communication import Serial
from .log import log
from .communicator import make_communicator
from .robot import make_robot


class Manager(EventEmitter):

    def __init__(self):
        super(Manager, self).__init__()

        self.Commmunicator = make_communicator(self)
        self.Robot = make_robot(self)

        self.communicators = []
        self.scanning_robots = {}

        self.serial = None

    def start(self, serial_address):
        if self.serial is not None:
            return

        self.serial = Serial(serial_address)

        self.communicators = []
        self.scanning_robots = {}

        self.serial.start()
        time.sleep(0.2)

        try:
            self.serial.write(OutgoingUSBMessageType.ResetBLE)

            self.serial.on('data', self.Commmunicator.handle_packet)
            self.serial.on('error', lambda error: self.emit('error', error))
            self.serial.on('closed', lambda closed: self.emit('closed', closed))

            time.sleep(0.2)
        except Exception as e:
            self.serial = None
            raise e

    def stop(self):
        self.serial.write(OutgoingUSBMessageType.ResetBLE)
        self.serial.stop()

    def start_scanning(self, timeout=2):
        self.scanning_robots = {}

        self.serial.write(OutgoingUSBMessageType.BeginScanning)

        time.sleep(timeout / 1000)

        self.serial.write(OutgoingUSBMessageType.EndScanning)

    def reset_ble(self):
        self.serial.write(OutgoingUSBMessageType.ResetBLE)

    def get_communicator(self, mac=None, connection_handle=None):
        if mac is None and connection_handle is None:
            raise APIError('Must specify mac or connection_handle')

        if connection_handle is not None:
            for communicator in self.communicators:
                if communicator.connection_handle == connection_handle:
                    return communicator

        if mac is not None:
            if not type(mac) == str:
                mac = ':' .join(str(i) for i in mac)

            for communicator in self.communicators:
                if communicator.mac_str == mac:
                    return communicator

        return None
